/*
 * Copyright (c) 2002 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: access.h,v 1.3 2002/01/14 10:21:31 pva Exp $
 * declaration of lowlevel functions for port access
 */

#ifndef ACCESS_H__
#define ACCESS_H__

#include <sys/param.h>
#include <sys/ioctl.h>

#ifdef linux
#include <sys/io.h>
#include <linux/videodev.h>
#elif defined __FreeBSD__
#include <machine/ioctl_bt848.h>
#include <fcntl.h>
#include <machine/cpufunc.h>
#elif defined __OpenBSD__ || defined __NetBSD__
#if defined __NetBSD__ 
#include <dev/ic/bt8xx.h>
#elif defined __OpenBSD__
#if OpenBSD < 200105
#include <machine/ioctl_bt848.h>
#else
#include <dev/ic/bt8xx.h>
#endif /* OpenBSD < 200105 */
#endif /* __NetBSD__ */
#include <machine/pio.h>
#include <machine/sysarch.h>
#endif

#include <err.h>
#include <unistd.h>

#include "io.h"

#ifdef linux
#define OUTL(a, b)	outl(b, a)
#define OUTW(a, b)	outw(b, a)
#define OUTB(a, b)	outb(b, a)
#else
#define OUTL(a, b)	outl(a, b)
#define OUTW(a, b)	outw(a, b)
#define OUTB(a, b)	outb(a, b)
#endif /* linux */

int radio_get_iopl(void);
int radio_release_iopl(void);
int radio_get_ioperms(unsigned long, int);
int radio_release_ioperms(unsigned long, int);

int radio_device_get(const char *, const char *, int);
int radio_device_release(int, const char *);

#endif /* ACCESS_H__ */
