/*
 * Copyright (c) 2002 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: bsdradio.c,v 1.2 2002/01/17 06:04:07 pva Exp $
 * Driver for OpenBSD and NetBSD radio driver
 */

#include "access.h"

#ifdef BSDRADIO

#include <sys/radioio.h>

#include <fcntl.h>	/* O_RDWR */

static int fd = -1;
static struct radio_info ri;
static int volumestep = 0;

extern const char *radio_device_1;
extern const char *radio_device_2;

const char *riocsinfo = "RIOCSINFO";
const char *riocginfo = "RIOCGINFO";

/******************************************************************/

int
get_port_bsdradio(unsigned long port) {
	int cur_vol, i = 0;

	fd = radio_device_get(radio_device_1, radio_device_2, O_RDWR);
	if (fd < 0)
		return -1;

	if (ioctl(fd, RIOCGINFO, &ri) < 0) {
		warn(riocginfo);
		return -1;
	}

	/* Do all operation silently */
	ri.mute = 1;

	/*
	 * Determine volume step for the card
	 */

	/* Save old value */
	cur_vol = ri.volume;
	while (i < 255) {
		ri.volume = ++i;
		if (ioctl(fd, RIOCSINFO, &ri) < 0) {
			warn(riocsinfo);
			return -1;
		}

		if (ioctl(fd, RIOCGINFO, &ri) < 0) {
			warn(riocginfo);
			return -1;
		}

		if (ri.volume) {
			volumestep = ri.volume;
			break;
		}
	}
	/* Restor old volume */
	ri.volume = cur_vol;
	if (ioctl(fd, RIOCSINFO, &ri) < 0) {
		warn(riocsinfo);
		return -1;
	}

	return 0;
}

int
free_port_bsdradio(void) {
	ri.mute = 0;
	if (ioctl(fd, RIOCSINFO, &ri) < 0) {
		warn(riocsinfo);
		return -1;
	}

	return radio_device_release(fd, radio_device_1);
}

unsigned long
info_port_bsdradio(void) {
	return 0ul;
}

void
set_frequency_bsdradio(unsigned long frequency) {
	ri.freq = frequency * 10;
}

void
mute_bsdradio(void) {
	ri.volume = 0;
}

void
volumeup_bsdradio(void) {
	if (ri.volume < 255)
		ri.volume += volumestep;
}

void
volumedn_bsdradio(void) {
	if (ri.volume > 0)
		ri.volume -= volumestep;
}

int
state_bsdradio(void) {
	int result = 3;

	if (ri.caps & RADIO_CAPS_DETECT_SIGNAL)
		if (ri.info & RADIO_INFO_SIGNAL)
			result -= 1;
	if (ri.caps & RADIO_CAPS_DETECT_STEREO)
		if (ri.info & RADIO_INFO_STEREO)
			result -= 2;

	return result;
}

void
mono_bsdradio(void) {
	ri.stereo = 0;
}

unsigned long
get_frequency_bsdradio(void) {
	return ri.freq;
}

int
get_volume_bsdradio(void) {
	return ri.volume;
}
#endif /* BSDRADIO */
