/*
 * Copyright (c) 2001 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gemtek-isa.c,v 1.3 2002/01/14 10:21:34 pva Exp $
 *
 * lowlevel io functions for fmio utility -- Gemtek ISA driver
 */

#include "access.h"
#include "bu2614.h"

static struct bu2614_t card;

/*********************************************************************/

int
Grab_Port_gemtek(unsigned long port) {
	int i, a, lasta;

	card.port = port;
	card.wren = (1 << 2);
	card.data = (1 << 1);
	card.clck = (1 << 0);

	if (radio_get_ioperms(card.port, 4) < 0)
		return -1;

	lasta = inb(card.port);

	/* ISA reads back 0xff when no card here */
	if (lasta == 0xff)
		return -1;

	/*
	 * Gemtek read as 0x3F, 0x1F, 0x37 or 0x17
	 * depending on signal state
	 */
	if ((lasta & 0x17) != 0x17)
		return -1;

	for (i = 1; i < 3; i++) {
		a = inb(card.port + i);
		/* Gemtek read back identical for 4 registers */
		if (lasta != a)
			return -1;
	}

	return 0;
}

int
Release_Port_gemtek(void) {
	return radio_release_ioperms(card.port, 4);
}

unsigned long
info_port_gemtek(void) {
	return card.port;
}

void
mute_gemtek(void) {
	OUTB(card.port, 0x10);
}

void
volumeup_gemtek(void) {
	OUTB(card.port, 0x20);
}

void
Change_Frequency_gemtek(unsigned long freq) {

	freq  = bu2614_conv_freq(freq);
	freq |= BU2614_BAND_FM | BU2614_REF_FREQ_6P25_KHZ | BU2614_GT_ON;
	bu2614_write(&card, freq);
	
	return;
}

int
state_gemtek(void) {
	usleep(50);
	return inb(card.port) & 8 ? 3 : 0;
}
