/*
 * Copyright (c) 2000 - 2002 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: io.h,v 1.63 2002/01/18 10:55:49 pva Exp $
 */

#ifndef IO_H__
#define IO_H__

#include <sys/types.h>

#if defined __FreeBSD__ || defined __OpenBSD__ || defined __NetBSD__
#define BSDBKTR
#endif /* __FreeBSD__ || __OpenBSD__ || __NetBSD__ */
#if defined __OpenBSD__ || defined __NetBSD__
#define BSDRADIO
#endif /* __OpenBSD__ || __NetBSD__ */

#define AFC_DELAY	300000

#define SEARCH_PROBE	15
#define SEARCH_LENGTH	19

#define TEST_FREQ	10630

typedef struct _radiocard_t {
	char *shname; /* Short name of driver */
#define MAX_SHNAME_LEN	31
	unsigned *ports;
	int info;
#define INFO_ROOT	0x80 /* driver needs root privs */
#define INFO_SCAN	0x40 /* detection is done by scanning */
#define INFO_FREQ	0x20 /* can determine current frequency */
#define INFO_VOLU	0x10 /* can determine current volume */
#define INFO_VARS	0x0F /* number of ports in the driver */
	char *name;	/* Name of card */
	unsigned lfreq;	/* Minimal allowed frequency */
	unsigned hfreq;	/* Maximal allowed frequency */
	int (*get_port)(unsigned long);
	int (*free_port)(void);
	unsigned long (*info_port)(void);
	unsigned long (*get_frequency)(void);
	int (*get_volume)(void);
	void (*set_frequency)(unsigned long);
	void (*mute)(void);
	void (*volume_up)(void);
	void (*volume_down)(void);
	int (*state)(void);
	void (*mono)(void);
	unsigned long (*search_up)(unsigned long);
	unsigned long (*search_down)(unsigned long);
} radiocard_t;

/* Aztech */
int Grab_Port_aztech(unsigned long);
int Release_Port_aztech(void);
unsigned long info_port_aztech(void);
void Change_Frequency_aztech(unsigned long);
void mute_aztech(void);
void volumeup_aztech(void);
void volumedn_aztech(void);
int state_aztech(void);
void mono_aztech(void);

/* Zoltrix */
int Grab_Port_zoltrix(unsigned long);
int Release_Port_zoltrix(void);
unsigned long info_port_zoltrix(void);
void Change_Frequency_zoltrix(unsigned long);
void mute_zoltrix(void);
void volumeup_zoltrix(void);
void volumedn_zoltrix(void);
int state_zoltrix(void);
void mono_zoltrix(void);

/* RadioTrack */
int Grab_Port_radiotrack(unsigned long);
int Release_Port_radiotrack(void);
unsigned long info_port_radiotrack(void);
void Change_Frequency_radiotrack(unsigned long);
void mute_radiotrack(void);
void volumeup_radiotrack(void);
void volumedn_radiotrack(void);
int state_radiotrack(void);
void mono_radiotrack(void);

/* GemTek ISA */
int Grab_Port_gemtek(unsigned long);
int Release_Port_gemtek(void);
unsigned long info_port_gemtek(void);
void Change_Frequency_gemtek(unsigned long);
void mute_gemtek(void);
void volumeup_gemtek(void);
int state_gemtek(void);

/* RadioTrack II */
int Grab_Port_radiotrackII(unsigned long);
int Release_Port_radiotrackII(void);
unsigned long info_port_radiotrackII(void);
void Change_Frequency_radiotrackII(unsigned long);
void mute_radiotrackII(void);
void unmute_radiotrackII(void);
int state_radiotrackII(void);
void mono_radiotrackII(void);
unsigned long search_up_radiotrackII(unsigned long);
unsigned long search_down_radiotrackII(unsigned long);

/* Brooktree Bt848 Driver */
int Grab_Port_bktr(unsigned long);
int Release_Port_bktr(void);
unsigned long info_port_bktr(void);
void Change_Frequency_bktr(unsigned long);
void mute_bktr(void);
void volumeup_bktr(void);
void volumedn_bktr(void);
int state_bktr(void);
void mono_bktr(void);
unsigned long get_freq_bktr(void);
int get_vol_bktr(void);

/* SoundForte RadioLink SF64-PCR */
int Grab_Port_sf64pcr(unsigned long);
int Release_Port_sf64pcr(void);
unsigned long info_port_sf64pcr(void);
unsigned long get_frequency_sf64pcr(void);
void set_frequency_sf64pcr(unsigned long);
void mute_sf64pcr(void);
void unmute_sf64pcr(void);
int state_sf64pcr(void);
void mono_sf64pcr(void);

/* SoundForte Quad X-treme SF256-PCP-R */
int Grab_Port_sf256pcpr(unsigned long);
int Release_Port_sf256pcpr(void);
unsigned long info_port_sf256pcpr(void);
unsigned long get_frequency_sf256pcpr(void);
void set_frequency_sf256pcpr(unsigned long);
void mute_sf256pcpr(void);
void unmute_sf256pcpr(void);
int state_sf256pcpr(void);
void mono_sf256pcpr(void);
unsigned long search_up_sf256pcpr(unsigned long);
unsigned long search_down_sf256pcpr(unsigned long);

/* SoundForte Theatre X-treme SF256-PCS */
int Grab_Port_sf256pcs(unsigned long);
int Release_Port_sf256pcs(void);
unsigned long info_port_sf256pcs(void);
unsigned long get_frequency_sf256pcs(void);
void set_frequency_sf256pcs(unsigned long);
void mute_sf256pcs(void);
void unmute_sf256pcs(void);
int state_sf256pcs(void);
void mono_sf256pcs(void);
unsigned long search_up_sf256pcs(unsigned long);
unsigned long search_down_sf256pcs(unsigned long);

/* SoundForte Legacy 128 SF16-FMD2 */
int Grab_Port_sf16fmd2(unsigned long);
int Release_Port_sf16fmd2(void);
unsigned long info_port_sf16fmd2(void);
void Change_Frequency_sf16fmd2(unsigned long);
void mute_sf16fmd2(void);
void volumeup_sf16fmd2(void);
void volumedn_sf16fmd2(void);
int state_sf16fmd2(void);
void mono_sf16fmd2(void);

/* SoundForte RadioLink SF16-FMR */
int Grab_Port_sf16fmr(unsigned long);
int Release_Port_sf16fmr(void);
unsigned long info_port_sf16fmr(void);
unsigned long get_freq_sf16fmr(void);
void Change_Frequency_sf16fmr(unsigned long);
void mute_sf16fmr(void);
void volumeup_sf16fmr(void);
void volumedn_sf16fmr(void);
int state_sf16fmr(void);

/* BMC HMA Radio card */
int Grab_Port_bmc(unsigned long);
int Release_Port_bmc(void);
unsigned long info_port_bmc(void);
unsigned long get_freq_bmc(void);
void Change_Frequency_bmc(unsigned long);
void mute_bmc(void);
void volumeup_bmc(void);
void volumedn_bmc(void);
int state_bmc(void);
void set_mono_bmc(void);

/* SoundForte RadioLink SF16-FMR2 */
int Grab_Port_sf16fmr2(unsigned long);
int Release_Port_sf16fmr2(void);
unsigned long info_port_sf16fmr2(void);
unsigned long get_frequency_sf16fmr2(void);
void set_frequency_sf16fmr2(unsigned long);
void mute_sf16fmr2(void);
void unmute_sf16fmr2(void);
void volumeup_sf16fmr2(void);
void volumedown_sf16fmr2(void);
int state_sf16fmr2(void);
void mono_sf16fmr2(void);
unsigned long search_up_sf16fmr2(unsigned long);
unsigned long search_down_sf16fmr2(unsigned long);

/* terratec */
int Grab_Port_tt(unsigned long);
int Release_Port_tt(void);
unsigned long info_port_tt(void);
void set_frequency_tt(unsigned long);
void mute_tt(void);
void volumeup_tt(void);
void volumedown_tt(void);
int state_tt(void);
void mono_tt(void);
unsigned long search_up_tt(unsigned long);
unsigned long search_down_tt(unsigned long);

/* Gemtek PCI */
int Grab_Port_gtp(unsigned long);
int Release_Port_gtp(void);
unsigned long info_port_gtp(void);
void Change_Frequency_gtp(unsigned long);
void mute_gtp(void);
void unmute_gtp(void);
int state_gtp(void);
void mono_gtp(void);
unsigned long search_up_gtp(unsigned long);
unsigned long search_down_gtp(unsigned long);

/* AIMS Lab Highway Xtreme */
#ifdef BSDBKTR
int get_port_xtreme(unsigned long);
int free_port_xtreme(void);
unsigned long info_port_xtreme(void);
void set_frequency_xtreme(unsigned long);
void mute_xtreme(void);
void unmute_xtreme(void);
int state_xtreme(void);
void mono_xtreme(void);
#endif /* BSDBKTR */

#ifdef BSDRADIO
int get_port_bsdradio(unsigned long);
int free_port_bsdradio(void);
unsigned long info_port_bsdradio(void);
unsigned long get_frequency_bsdradio(void);
void set_frequency_bsdradio(unsigned long);
void mute_bsdradio(void);
void volumeup_bsdradio(void);
void volumedn_bsdradio(void);
int get_volume_bsdradio(void);
int state_bsdradio(void);
void mono_bsdradio(void);
#endif /* BSDRADIO */

void range(unsigned, unsigned *, unsigned *, unsigned);
void draw_stick(int);
void mono_stub(void);
unsigned long search_stub(unsigned long);
unsigned long get_freq_stub(void);
int get_vol_stub(void);

/* Find address of a PCI FM tuner card */
u_int32_t pci_bus_locate(u_int16_t, u_int16_t);

void print_w(const char *);
void print_w2(const char *, const char *);
void print_wx(const char *);
void print_wx2(const char *, const char *);

#endif /* IO_H__ */
